@@ -37,6 +37,7 @@ use crate::properties::{
     Overflow, PaintTarget, ShapeRendering, StrokeLinecap, StrokeLinejoin, TextRendering,
 };
 use crate::rect::{IRect, Rect};
+use crate::session::Session;
 use crate::surface_utils::{
     shared_surface::ExclusiveImageSurface, shared_surface::SharedImageSurface,
     shared_surface::SurfaceType,
@@ -107,7 +108,7 @@ pub struct FontOptions {
 #[derive(Debug, Copy, Clone, PartialEq)]
 pub enum ClipMode {
     ClipToViewport,
-    ClipToVbox,
+    NoClip,
 }
 
 /// Set path on the cairo context, or clear it.
@@ -169,6 +170,8 @@ struct Viewport {
 }
 
 pub struct DrawingCtx {
+    session: Session,
+
     initial_viewport: Viewport,
 
     dpi: Dpi,
@@ -196,6 +199,7 @@ pub enum DrawingMode {
 ///
 /// This creates a DrawingCtx internally and starts drawing at the specified `node`.
 pub fn draw_tree(
+    session: Session,
     mode: DrawingMode,
     cr: &cairo::Context,
     viewport: Rect,
@@ -239,6 +243,7 @@ pub fn draw_tree(
     let viewport = viewport.translate((-viewport.x0, -viewport.y0));
 
     let mut draw_ctx = DrawingCtx::new(
+        session,
         cr,
         transform,
         viewport,
@@ -281,6 +286,7 @@ const CAIRO_TAG_LINK: &str = "Link";
 
 impl DrawingCtx {
     fn new(
+        session: Session,
         cr: &cairo::Context,
         transform: Transform,
         viewport: Rect,
@@ -296,6 +302,7 @@ impl DrawingCtx {
         let viewport_stack = vec![initial_viewport];
 
         DrawingCtx {
+            session,
             initial_viewport,
             dpi,
             cr_stack: Rc::new(RefCell::new(Vec::new())),
@@ -320,6 +327,7 @@ impl DrawingCtx {
         cr_stack.borrow_mut().push(self.cr.clone());
 
         DrawingCtx {
+            session: self.session.clone(),
             initial_viewport: self.initial_viewport,
             dpi: self.dpi,
             cr_stack,
@@ -332,6 +340,10 @@ impl DrawingCtx {
         }
     }
 
+    pub fn session(&self) -> &Session {
+        &self.session
+    }
+
     pub fn user_language(&self) -> &UserLanguage {
         &self.user_language
     }
@@ -474,9 +486,9 @@ impl DrawingCtx {
         vbox: Option<ViewBox>,
         viewport: Rect,
         preserve_aspect_ratio: AspectRatio,
-        clip_mode: Option<ClipMode>,
+        clip_mode: ClipMode,
     ) -> Option<ViewParams> {
-        if let Some(ClipMode::ClipToViewport) = clip_mode {
+        if let ClipMode::ClipToViewport = clip_mode {
             clip_to_rectangle(&self.cr, &viewport);
         }
 
@@ -489,6 +501,7 @@ impl DrawingCtx {
                     ),
                     Some(v) => {
                         rsvg_log!(
+                            self.session,
                             "ignoring viewBox ({}, {}, {}, {}) since it is not usable",
                             v.x0,
                             v.y0,
@@ -502,12 +515,6 @@ impl DrawingCtx {
             .map(|t| {
                 self.cr.transform(t.into());
 
-                if let Some(vbox) = vbox {
-                    if let Some(ClipMode::ClipToVbox) = clip_mode {
-                        clip_to_rectangle(&self.cr, &*vbox);
-                    }
-                }
-
                 let top_viewport = self.get_top_viewport();
 
                 self.push_viewport(Viewport {
@@ -575,7 +582,7 @@ impl DrawingCtx {
             Ok(n) => n,
 
             Err(AcquireError::CircularReference(_)) => {
-                rsvg_log!("circular reference in element {}", mask_node);
+                rsvg_log!(self.session, "circular reference in element {}", mask_node);
                 return Ok(None);
             }
 
@@ -641,8 +648,13 @@ impl DrawingCtx {
 
             let mut mask_draw_ctx = self.nested(mask_cr);
 
-            let stacking_ctx =
-                StackingContext::new(acquired_nodes, &mask_element, Transform::identity(), values);
+            let stacking_ctx = StackingContext::new(
+                self.session(),
+                acquired_nodes,
+                &mask_element,
+                Transform::identity(),
+                values,
+            );
 
             let res = mask_draw_ctx.with_discrete_layer(
                 &stacking_ctx,
@@ -793,6 +805,7 @@ impl DrawingCtx {
                                         current_color,
                                         None,
                                         None,
+                                        self.session(),
                                     )
                                     .to_user_space(&bbox, &params, values),
                             );
@@ -807,6 +820,7 @@ impl DrawingCtx {
                                         current_color,
                                         None,
                                         None,
+                                        self.session(),
                                     )
                                     .to_user_space(&bbox, &params, values),
                             );
@@ -1036,7 +1050,7 @@ impl DrawingCtx {
             Ok(n) => n,
 
             Err(AcquireError::CircularReference(ref node)) => {
-                rsvg_log!("circular reference in element {}", node);
+                rsvg_log!(self.session, "circular reference in element {}", node);
                 return Ok(false);
             }
 
@@ -1096,6 +1110,7 @@ impl DrawingCtx {
                     let elt = pattern_node.borrow_element();
 
                     let stacking_ctx = StackingContext::new(
+                        self.session(),
                         acquired_nodes,
                         &elt,
                         Transform::identity(),
@@ -1341,9 +1356,9 @@ impl DrawingCtx {
             || image.overflow == Overflow::Visible)
             && image.aspect.is_slice()
         {
-            Some(ClipMode::ClipToViewport)
+            ClipMode::ClipToViewport
         } else {
-            None
+            ClipMode::NoClip
         };
 
         // The bounding box for <image> is decided by the values of the image's x, y, w, h
@@ -1611,7 +1626,7 @@ impl DrawingCtx {
             Ok(n) => n,
 
             Err(AcquireError::CircularReference(_)) => {
-                rsvg_log!("circular reference in element {}", node);
+                rsvg_log!(self.session, "circular reference in element {}", node);
                 return Ok(self.empty_bbox());
             }
 
@@ -1622,7 +1637,7 @@ impl DrawingCtx {
             Ok(acquired) => acquired,
 
             Err(AcquireError::CircularReference(node)) => {
-                rsvg_log!("circular reference in element {}", node);
+                rsvg_log!(self.session, "circular reference in element {}", node);
                 return Ok(self.empty_bbox());
             }
 
@@ -1635,7 +1650,12 @@ impl DrawingCtx {
             Err(AcquireError::InvalidLinkType(_)) => unreachable!(),
 
             Err(AcquireError::LinkNotFound(node_id)) => {
-                rsvg_log!("element {} references nonexistent \"{}\"", node, node_id);
+                rsvg_log!(
+                    self.session,
+                    "element {} references nonexistent \"{}\"",
+                    node,
+                    node_id
+                );
                 return Ok(self.empty_bbox());
             }
         };
@@ -1664,18 +1684,22 @@ impl DrawingCtx {
             let elt = child.borrow_element();
 
             let symbol = borrow_element_as!(child, Symbol);
+            let symbol_values = elt.get_computed_values();
 
-            let clip_mode = if !values.is_overflow()
-                || (values.overflow() == Overflow::Visible
-                    && elt.get_specified_values().is_overflow())
-            {
-                Some(ClipMode::ClipToVbox)
+            // FIXME: do we need to look at preserveAspectRatio.slice, like in draw_image()?
+            let clip_mode = if !symbol_values.is_overflow() {
+                ClipMode::ClipToViewport
             } else {
-                None
+                ClipMode::NoClip
             };
 
-            let stacking_ctx =
-                StackingContext::new(acquired_nodes, &use_element, Transform::identity(), values);
+            let stacking_ctx = StackingContext::new(
+                self.session(),
+                acquired_nodes,
+                &use_element,
+                Transform::identity(),
+                values,
+            );
 
             self.with_discrete_layer(
                 &stacking_ctx,
@@ -1708,6 +1732,7 @@ impl DrawingCtx {
             // otherwise the referenced node is not a <symbol>; process it generically
 
             let stacking_ctx = StackingContext::new(
+                self.session(),
                 acquired_nodes,
                 &use_element,
                 Transform::new_translate(use_rect.x0, use_rect.y0),
