@@ -313,12 +313,6 @@ pub enum AllowedUrlError {
     /// or in one directory below the base file.
     NotSiblingOrChildOfBaseFile,
 
-    /// Loaded file:// URLs cannot have a query part, e.g. `file:///foo?blah`
-    NoQueriesAllowed,
-
-    /// URLs may not have fragment identifiers at this stage
-    NoFragmentIdentifierAllowed,
-
     /// Error when obtaining the file path or the base file path
     InvalidPath,
 
@@ -331,18 +325,17 @@ pub enum AllowedUrlError {
 
 impl fmt::Display for AllowedUrlError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        use AllowedUrlError::*;
         match *self {
-            UrlParseError(e) => write!(f, "URL parse error: {}", e),
-            BaseRequired => write!(f, "base required"),
-            DifferentUriSchemes => write!(f, "different URI schemes"),
-            DisallowedScheme => write!(f, "disallowed scheme"),
-            NotSiblingOrChildOfBaseFile => write!(f, "not sibling or child of base file"),
-            NoQueriesAllowed => write!(f, "no queries allowed"),
-            NoFragmentIdentifierAllowed => write!(f, "no fragment identifier allowed"),
-            InvalidPath => write!(f, "invalid path"),
-            BaseIsRoot => write!(f, "base is root"),
-            CanonicalizationError => write!(f, "canonicalization error"),
+            AllowedUrlError::UrlParseError(e) => write!(f, "URL parse error: {}", e),
+            AllowedUrlError::BaseRequired => write!(f, "base required"),
+            AllowedUrlError::DifferentUriSchemes => write!(f, "different URI schemes"),
+            AllowedUrlError::DisallowedScheme => write!(f, "disallowed scheme"),
+            AllowedUrlError::NotSiblingOrChildOfBaseFile => {
+                write!(f, "not sibling or child of base file")
+            }
+            AllowedUrlError::InvalidPath => write!(f, "invalid path"),
+            AllowedUrlError::BaseIsRoot => write!(f, "base is root"),
+            AllowedUrlError::CanonicalizationError => write!(f, "canonicalization error"),
         }
     }
 }
@@ -443,6 +436,15 @@ pub enum ImplementationLimit {
     /// allow loading more than a certain number of elements during
     /// the initial loading process.
     TooManyLoadedElements,
+
+    /// Document exceeded the number of attributes that can be attached to
+    /// an element.
+    ///
+    /// This is here because librsvg uses u16 to address attributes. It should
+    /// be essentially impossible to actually hit this limit, because the
+    /// number of attributes that the SVG standard ascribes meaning to are
+    /// lower than this limit.
+    TooManyAttributes,
 }
 
 impl error::Error for LoadingError {}
@@ -493,6 +495,12 @@ impl fmt::Display for ImplementationLimit {
                 "cannot load more than {} XML elements",
                 limits::MAX_LOADED_ELEMENTS
             ),
+
+            ImplementationLimit::TooManyAttributes => write!(
+                f,
+                "cannot load more than {} XML attributes",
+                limits::MAX_LOADED_ATTRIBUTES
+            ),
         }
     }
 }
