@@ -26,6 +26,7 @@ use crate::css::{DeclParser, Declaration, Origin};
 use crate::error::*;
 use crate::parsers::{Parse, ParseValue};
 use crate::property_macros::Property;
+use crate::session::Session;
 use crate::transform::{Transform, TransformAttribute, TransformProperty};
 use crate::xml::Attributes;
 
@@ -805,7 +806,7 @@ impl SpecifiedValues {
         }
     }
 
-    fn parse_one_presentation_attribute(&mut self, attr: QualName, value: &str) {
+    fn parse_one_presentation_attribute(&mut self, session: &Session, attr: QualName, value: &str) {
         let mut input = ParserInput::new(value);
         let mut parser = Parser::new(&mut input);
 
@@ -815,6 +816,7 @@ impl SpecifiedValues {
                     self.set_parsed_property(&prop);
                 } else {
                     rsvg_log!(
+                        session,
                         "(ignoring invalid presentation attribute {:?}\n    value=\"{}\")\n",
                         attr.expanded(),
                         value,
@@ -839,6 +841,7 @@ impl SpecifiedValues {
 
                 t.to_css(&mut tok).unwrap(); // FIXME: what do we do with a fmt::Error?
                 rsvg_log!(
+                    session,
                     "(ignoring invalid presentation attribute {:?}\n    value=\"{}\"\n    \
                      unexpected token '{}')",
                     attr.expanded(),
@@ -852,6 +855,7 @@ impl SpecifiedValues {
                 ..
             }) => {
                 rsvg_log!(
+                    session,
                     "(ignoring invalid presentation attribute {:?}\n    value=\"{}\"\n    \
                      unexpected end of input)",
                     attr.expanded(),
@@ -864,6 +868,7 @@ impl SpecifiedValues {
                 ..
             }) => {
                 rsvg_log!(
+                    session,
                     "(ignoring invalid presentation attribute {:?}\n    value=\"{}\"\n    \
                      unexpected error)",
                     attr.expanded(),
@@ -876,6 +881,7 @@ impl SpecifiedValues {
                 ..
             }) => {
                 rsvg_log!(
+                    session,
                     "(ignoring invalid presentation attribute {:?}\n    value=\"{}\"\n    {})",
                     attr.expanded(),
                     value,
@@ -887,6 +893,7 @@ impl SpecifiedValues {
 
     pub fn parse_presentation_attributes(
         &mut self,
+        session: &Session,
         attrs: &Attributes,
     ) -> Result<(), ElementError> {
         for (attr, value) in attrs.iter() {
@@ -918,7 +925,7 @@ impl SpecifiedValues {
                     )));
                 }
 
-                _ => self.parse_one_presentation_attribute(attr, value),
+                _ => self.parse_one_presentation_attribute(session, attr, value),
             }
         }
 
@@ -951,6 +958,7 @@ impl SpecifiedValues {
         declarations: &str,
         origin: Origin,
         important_styles: &mut HashSet<QualName>,
+        session: &Session,
     ) -> Result<(), ElementError> {
         let mut input = ParserInput::new(declarations);
         let mut parser = Parser::new(&mut input);
@@ -959,7 +967,7 @@ impl SpecifiedValues {
             .filter_map(|r| match r {
                 Ok(decl) => Some(decl),
                 Err(e) => {
-                    rsvg_log!("Invalid declaration; ignoring: {:?}", e);
+                    rsvg_log!(session, "Invalid declaration; ignoring: {:?}", e);
                     None
                 }
             })
@@ -1115,7 +1123,7 @@ mod tests {
         let mut specified = SpecifiedValues::default();
 
         assert!(specified
-            .parse_style_declarations("", Origin::Author, &mut HashSet::new())
+            .parse_style_declarations("", Origin::Author, &mut HashSet::new(), &Session::default())
             .is_ok())
     }
 }
