@@ -16,6 +16,7 @@ use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::parsers::{Parse, ParseValue};
 use crate::properties::ComputedValues;
 use crate::rect::Rect;
+use crate::session::Session;
 use crate::viewbox::*;
 use crate::xml::Attributes;
 
@@ -36,7 +37,13 @@ impl Draw for Group {
         let values = cascaded.get();
 
         let elt = node.borrow_element();
-        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, values.transform(), values);
+        let stacking_ctx = StackingContext::new(
+            draw_ctx.session(),
+            acquired_nodes,
+            &elt,
+            values.transform(),
+            values,
+        );
 
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
@@ -77,7 +84,13 @@ impl Draw for Switch {
         let values = cascaded.get();
 
         let elt = node.borrow_element();
-        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, values.transform(), values);
+        let stacking_ctx = StackingContext::new(
+            draw_ctx.session(),
+            acquired_nodes,
+            &elt,
+            values.transform(),
+            values,
+        );
 
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
@@ -213,10 +226,11 @@ impl Svg {
 
         let has_parent = node.parent().is_some();
 
+        // FIXME: do we need to look at preserveAspectRatio.slice, like in DrawingCtx::draw_image()?
         let clip_mode = if !values.is_overflow() && has_parent {
-            Some(ClipMode::ClipToViewport)
+            ClipMode::ClipToViewport
         } else {
-            None
+            ClipMode::NoClip
         };
 
         let svg_viewport = self.get_viewport(&params, values, !has_parent);
@@ -252,7 +266,7 @@ impl Svg {
 }
 
 impl SetAttributes for Svg {
-    fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
                 expanded_name!("", "preserveAspectRatio") => {
@@ -279,7 +293,13 @@ impl Draw for Svg {
         let values = cascaded.get();
 
         let elt = node.borrow_element();
-        let stacking_ctx = StackingContext::new(acquired_nodes, &elt, values.transform(), values);
+        let stacking_ctx = StackingContext::new(
+            draw_ctx.session(),
+            acquired_nodes,
+            &elt,
+            values.transform(),
+            values,
+        );
 
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
@@ -327,7 +347,7 @@ impl Default for Use {
 }
 
 impl SetAttributes for Use {
-    fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
                 ref a if is_href(a) => set_href(
@@ -368,6 +388,7 @@ impl Draw for Use {
                 values.color().0,
                 cascaded.context_fill.clone(),
                 cascaded.context_stroke.clone(),
+                draw_ctx.session(),
             );
 
             let fill_paint = values.fill().0.resolve(
@@ -376,6 +397,7 @@ impl Draw for Use {
                 values.color().0,
                 cascaded.context_fill.clone(),
                 cascaded.context_stroke.clone(),
+                draw_ctx.session(),
             );
 
             draw_ctx.draw_from_use_node(
@@ -411,7 +433,7 @@ impl Symbol {
 }
 
 impl SetAttributes for Symbol {
-    fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
                 expanded_name!("", "preserveAspectRatio") => {
@@ -442,7 +464,7 @@ impl ClipPath {
 }
 
 impl SetAttributes for ClipPath {
-    fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
         let result = attrs
             .iter()
             .find(|(attr, _)| attr.expanded() == expanded_name!("", "clipPathUnits"))
@@ -505,7 +527,7 @@ impl Mask {
 }
 
 impl SetAttributes for Mask {
-    fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
                 expanded_name!("", "x") => self.x = attr.parse(value)?,
@@ -530,7 +552,7 @@ pub struct Link {
 }
 
 impl SetAttributes for Link {
-    fn set_attributes(&mut self, attrs: &Attributes) -> ElementResult {
+    fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) -> ElementResult {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
                 ref a if is_href(a) => set_href(a, &mut self.link, value.to_owned()),
@@ -573,6 +595,7 @@ impl Draw for Link {
         };
 
         let stacking_ctx = StackingContext::new_with_link(
+            draw_ctx.session(),
             acquired_nodes,
             &elt,
             values.transform(),
