@@ -1,13 +1,13 @@
 //! Determine which URLs are allowed for loading.
 
 use std::fmt;
+use std::io;
 use std::ops::Deref;
+use std::path::{Path, PathBuf};
 use url::Url;
 
 use crate::error::AllowedUrlError;
 
-/// Decides which URLs are allowed to be loaded.
-///
 /// Currently only contains the base URL.
 ///
 /// The plan is to add:
@@ -29,11 +29,6 @@ impl UrlResolver {
         UrlResolver { base_url }
     }
 
-    /// Decides which URLs are allowed to be loaded based on the presence of a base URL.
-    ///
-    /// This function implements the policy described in "Security and locations of
-    /// referenced files" in the [crate
-    /// documentation](index.html#security-and-locations-of-referenced-files).
     pub fn resolve_href(&self, href: &str) -> Result<AllowedUrl, AllowedUrlError> {
         let url = Url::options()
             .base_url(self.base_url.as_ref())
@@ -45,17 +40,6 @@ impl UrlResolver {
             return Ok(AllowedUrl(url));
         }
 
-        // Queries are not allowed.
-        if url.query().is_some() {
-            return Err(AllowedUrlError::NoQueriesAllowed);
-        }
-
-        // Fragment identifiers are not allowed.  They should have been stripped
-        // upstream, by NodeId.
-        if url.fragment().is_some() {
-            return Err(AllowedUrlError::NoFragmentIdentifierAllowed);
-        }
-
         // All other sources require a base url
         if self.base_url.is_none() {
             return Err(AllowedUrlError::BaseRequired);
@@ -78,26 +62,6 @@ impl UrlResolver {
             return Err(AllowedUrlError::DisallowedScheme);
         }
 
-        // The rest of this function assumes file: URLs; guard against
-        // incorrect refactoring.
-        assert!(url.scheme() == "file");
-
-        // If we have a base_uri of "file:///foo/bar.svg", and resolve an href of ".",
-        // Url.parse() will give us "file:///foo/".  We don't want that, so check
-        // if the last path segment is empty - it will not be empty for a normal file.
-
-        if let Some(segments) = url.path_segments() {
-            if segments
-                .last()
-                .expect("URL path segments always contain at last 1 element")
-                .is_empty()
-            {
-                return Err(AllowedUrlError::NotSiblingOrChildOfBaseFile);
-            }
-        } else {
-            unreachable!("the file: URL cannot have an empty path");
-        }
-
         // We have two file: URIs.  Now canonicalize them (remove .. and symlinks, etc.)
         // and see if the directories match
 
@@ -115,17 +79,13 @@ impl UrlResolver {
 
         let base_parent = base_parent.unwrap();
 
-        let path_canon = url_path
-            .canonicalize()
-            .map_err(|_| AllowedUrlError::CanonicalizationError)?;
-        let parent_canon = base_parent
-            .canonicalize()
-            .map_err(|_| AllowedUrlError::CanonicalizationError)?;
+        let url_canon =
+            canonicalize(&url_path).map_err(|_| AllowedUrlError::CanonicalizationError)?;
+        let parent_canon =
+            canonicalize(&base_parent).map_err(|_| AllowedUrlError::CanonicalizationError)?;
 
-        if path_canon.starts_with(parent_canon) {
-            // Finally, convert the canonicalized path back to a URL.
-            let path_to_url = Url::from_file_path(path_canon).unwrap();
-            Ok(AllowedUrl(path_to_url))
+        if url_canon.starts_with(parent_canon) {
+            Ok(AllowedUrl(url))
         } else {
             Err(AllowedUrlError::NotSiblingOrChildOfBaseFile)
         }
@@ -156,12 +116,21 @@ impl fmt::Display for AllowedUrl {
     }
 }
 
+// For tests, we don't want to touch the filesystem.  In that case,
+// assume that we are being passed canonical file names.
+#[cfg(not(test))]
+fn canonicalize<P: AsRef<Path>>(path: P) -> Result<PathBuf, io::Error> {
+    path.as_ref().canonicalize()
+}
+#[cfg(test)]
+fn canonicalize<P: AsRef<Path>>(path: P) -> Result<PathBuf, io::Error> {
+    Ok(path.as_ref().to_path_buf())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    use std::path::PathBuf;
-
     #[test]
     fn disallows_relative_file_with_no_base_file() {
         let url_resolver = UrlResolver::new(None);
@@ -222,124 +191,66 @@ mod tests {
         );
     }
 
-    fn url_from_test_fixtures(filename_relative_to_librsvg_srcdir: &str) -> Url {
-        let path = PathBuf::from(filename_relative_to_librsvg_srcdir);
-        let absolute = path
-            .canonicalize()
-            .expect("files from test fixtures are supposed to canonicalize");
-        Url::from_file_path(absolute).unwrap()
-    }
-
     #[test]
     fn allows_relative() {
-        let base_url = url_from_test_fixtures("tests/fixtures/loading/bar.svg");
-        let url_resolver = UrlResolver::new(Some(base_url));
-
+        let url_resolver = UrlResolver::new(Some(
+            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
+        ));
         let resolved = url_resolver.resolve_href("foo.svg").unwrap();
-        let resolved_str = resolved.as_str();
-        assert!(resolved_str.ends_with("/loading/foo.svg"));
+        let expected = make_file_uri("/example/foo.svg");
+        assert_eq!(resolved.as_ref(), expected);
     }
 
     #[test]
     fn allows_sibling() {
-        let url_resolver = UrlResolver::new(Some(url_from_test_fixtures(
-            "tests/fixtures/loading/bar.svg",
-        )));
+        let url_resolver = UrlResolver::new(Some(
+            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
+        ));
         let resolved = url_resolver
-            .resolve_href(url_from_test_fixtures("tests/fixtures/loading/foo.svg").as_str())
+            .resolve_href(&make_file_uri("/example/foo.svg"))
             .unwrap();
-
-        let resolved_str = resolved.as_str();
-        assert!(resolved_str.ends_with("/loading/foo.svg"));
+        let expected = make_file_uri("/example/foo.svg");
+        assert_eq!(resolved.as_ref(), expected);
     }
 
     #[test]
     fn allows_child_of_sibling() {
-        let url_resolver = UrlResolver::new(Some(url_from_test_fixtures(
-            "tests/fixtures/loading/bar.svg",
-        )));
+        let url_resolver = UrlResolver::new(Some(
+            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
+        ));
         let resolved = url_resolver
-            .resolve_href(url_from_test_fixtures("tests/fixtures/loading/subdir/baz.svg").as_str())
+            .resolve_href(&make_file_uri("/example/subdir/foo.svg"))
             .unwrap();
-
-        let resolved_str = resolved.as_str();
-        assert!(resolved_str.ends_with("/loading/subdir/baz.svg"));
+        let expected = make_file_uri("/example/subdir/foo.svg");
+        assert_eq!(resolved.as_ref(), expected);
     }
 
-    // Ignore on Windows since we test for /etc/passwd
-    #[cfg(unix)]
     #[test]
     fn disallows_non_sibling() {
-        let url_resolver = UrlResolver::new(Some(url_from_test_fixtures(
-            "tests/fixtures/loading/bar.svg",
-        )));
+        let url_resolver = UrlResolver::new(Some(
+            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
+        ));
         assert!(matches!(
             url_resolver.resolve_href(&make_file_uri("/etc/passwd")),
             Err(AllowedUrlError::NotSiblingOrChildOfBaseFile)
         ));
     }
 
+    #[cfg(windows)]
     #[test]
-    fn disallows_queries() {
-        let url_resolver = UrlResolver::new(Some(
-            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
-        ));
-        assert!(matches!(
-            url_resolver.resolve_href(".?../../../../../../../../../../etc/passwd"),
-            Err(AllowedUrlError::NoQueriesAllowed)
-        ));
-    }
+    fn invalid_url_from_test_suite() {
+        // This is required for Url to panic.
+        let resolver =
+            UrlResolver::new(Some(Url::parse("file:///c:/foo.svg").expect("initial url")));
+        // With this, it doesn't panic:
+        //   let resolver = UrlResolver::new(None);
 
-    #[test]
-    fn disallows_weird_relative_uris() {
-        let url_resolver = UrlResolver::new(Some(
-            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
-        ));
-
-        assert!(url_resolver
-            .resolve_href(".@../../../../../../../../../../etc/passwd")
-            .is_err());
-        assert!(url_resolver
-            .resolve_href(".$../../../../../../../../../../etc/passwd")
-            .is_err());
-        assert!(url_resolver
-            .resolve_href(".%../../../../../../../../../../etc/passwd")
-            .is_err());
-        assert!(url_resolver
-            .resolve_href(".*../../../../../../../../../../etc/passwd")
-            .is_err());
-        assert!(url_resolver
-            .resolve_href("~/../../../../../../../../../../etc/passwd")
-            .is_err());
-    }
-
-    #[test]
-    fn disallows_dot_sibling() {
-        let url_resolver = UrlResolver::new(Some(
-            Url::parse(&make_file_uri("/example/bar.svg")).unwrap(),
-        ));
-
-        assert!(matches!(
-            url_resolver.resolve_href("."),
-            Err(AllowedUrlError::NotSiblingOrChildOfBaseFile)
-        ));
-        assert!(matches!(
-            url_resolver.resolve_href(".#../../../../../../../../../../etc/passwd"),
-            Err(AllowedUrlError::NoFragmentIdentifierAllowed)
-        ));
-    }
-
-    #[test]
-    fn disallows_fragment() {
-        // UrlResolver::resolve_href() explicitly disallows fragment identifiers.
-        // This is because they should have been stripped before calling that function,
-        // by NodeId or the Iri machinery.
-        let url_resolver =
-            UrlResolver::new(Some(Url::parse("https://example.com/foo.svg").unwrap()));
-
-        assert!(matches!(
-            url_resolver.resolve_href("bar.svg#fragment"),
-            Err(AllowedUrlError::NoFragmentIdentifierAllowed)
-        ));
+        // The following panics, when using a base URL
+        //   match resolver.resolve_href("file://invalid.css") {
+        // so, use a less problematic case, hopefully
+        match resolver.resolve_href("file://") {
+            Ok(_) => println!("yay!"),
+            Err(e) => println!("err: {}", e),
+        }
     }
 }
