@@ -16,11 +16,14 @@ use std::collections::HashMap;
 use std::rc::{Rc, Weak};
 use std::str;
 use std::string::ToString;
+use std::sync::Arc;
 use xml5ever::tendril::format_tendril;
 use xml5ever::tokenizer::{TagKind, Token, TokenSink, XmlTokenizer, XmlTokenizerOpts};
 
+use crate::css::{Origin, Stylesheet};
 use crate::document::{Document, DocumentBuilder};
 use crate::error::{ImplementationLimit, LoadingError};
+use crate::handle::LoadOptions;
 use crate::io::{self, IoError};
 use crate::limits::MAX_LOADED_ELEMENTS;
 use crate::node::{Node, NodeBorrow};
@@ -112,7 +115,7 @@ struct XmlStateInner {
 pub struct XmlState {
     inner: RefCell<XmlStateInner>,
 
-    unlimited_size: bool,
+    load_options: Arc<LoadOptions>,
 }
 
 /// Errors returned from XmlState::acquire()
@@ -135,7 +138,7 @@ impl XmlStateInner {
 }
 
 impl XmlState {
-    fn new(document_builder: DocumentBuilder, unlimited_size: bool) -> XmlState {
+    fn new(document_builder: DocumentBuilder, load_options: Arc<LoadOptions>) -> XmlState {
         XmlState {
             inner: RefCell::new(XmlStateInner {
                 weak: None,
@@ -146,7 +149,7 @@ impl XmlState {
                 entities: HashMap::new(),
             }),
 
-            unlimited_size,
+            load_options,
         }
     }
 
@@ -268,23 +271,49 @@ impl XmlState {
 
             let mut inner = self.inner.borrow_mut();
 
+            let session = inner.document_builder.as_mut().unwrap().session().clone();
+
+            if type_.as_deref() != Some("text/css")
+                || (alternate.is_some() && alternate.as_deref() != Some("no"))
+            {
+                rsvg_log!(
+                    session,
+                    "invalid parameters in XML processing instruction for stylesheet",
+                );
+                return;
+            }
+
             if let Some(href) = href {
-                // FIXME: https://www.w3.org/TR/xml-stylesheet/ does not seem to specify
-                // what to do if the stylesheet cannot be loaded, so here we ignore the error.
-                if inner
-                    .document_builder
-                    .as_mut()
-                    .unwrap()
-                    .append_stylesheet_from_xml_processing_instruction(alternate, type_, &href)
-                    .is_err()
-                {
+                if let Ok(aurl) = self.load_options.url_resolver.resolve_href(&href) {
+                    if let Ok(stylesheet) =
+                        Stylesheet::from_href(&aurl, Origin::Author, session.clone())
+                    {
+                        inner
+                            .document_builder
+                            .as_mut()
+                            .unwrap()
+                            .append_stylesheet(stylesheet);
+                    } else {
+                        // FIXME: https://www.w3.org/TR/xml-stylesheet/ does not seem to specify
+                        // what to do if the stylesheet cannot be loaded, so here we ignore the error.
+                        rsvg_log!(
+                            session,
+                            "could not create stylesheet from {} in XML processing instruction",
+                            href
+                        );
+                    }
+                } else {
                     rsvg_log!(
-                        "invalid xml-stylesheet {} in XML processing instruction",
+                        session,
+                        "{} not allowed for xml-stylesheet in XML processing instruction",
                         href
                     );
                 }
             } else {
-                rsvg_log!("xml-stylesheet processing instruction does not have href; ignoring");
+                rsvg_log!(
+                    session,
+                    "xml-stylesheet processing instruction does not have href; ignoring"
+                );
             }
         } else {
             self.error(LoadingError::XmlParseError(String::from(
@@ -378,7 +407,18 @@ impl XmlState {
                 .collect::<String>();
 
             let builder = inner.document_builder.as_mut().unwrap();
-            builder.append_stylesheet_from_text(&stylesheet_text);
+            let session = builder.session().clone();
+
+            if let Ok(stylesheet) = Stylesheet::from_data(
+                &stylesheet_text,
+                &self.load_options.url_resolver,
+                Origin::Author,
+                session.clone(),
+            ) {
+                builder.append_stylesheet(stylesheet);
+            } else {
+                rsvg_log!(session, "invalid inline stylesheet");
+            }
         }
     }
 
@@ -477,6 +517,15 @@ impl XmlState {
         encoding: Option<&str>,
     ) -> Result<(), AcquireError> {
         if let Some(href) = href {
+            let session = self
+                .inner
+                .borrow()
+                .document_builder
+                .as_ref()
+                .unwrap()
+                .session()
+                .clone();
+
             let aurl = self
                 .inner
                 .borrow()
@@ -487,7 +536,7 @@ impl XmlState {
                 .map_err(|e| {
                     // FIXME: should AlloweUrlError::UrlParseError be a fatal error,
                     // not a resource error?
-                    rsvg_log!("could not acquire \"{}\": {}", href, e);
+                    rsvg_log!(session, "could not acquire \"{}\": {}", href, e);
                     AcquireError::ResourceError
                 })?;
 
@@ -517,8 +566,17 @@ impl XmlState {
     }
 
     fn acquire_text(&self, aurl: &AllowedUrl, encoding: Option<&str>) -> Result<(), AcquireError> {
+        let session = self
+            .inner
+            .borrow()
+            .document_builder
+            .as_ref()
+            .unwrap()
+            .session()
+            .clone();
+
         let binary = io::acquire_data(aurl, None).map_err(|e| {
-            rsvg_log!("could not acquire \"{}\": {}", aurl, e);
+            rsvg_log!(session, "could not acquire \"{}\": {}", aurl, e);
             AcquireError::ResourceError
         })?;
 
@@ -577,9 +635,14 @@ impl XmlState {
             .unwrap()
             .upgrade()
             .unwrap();
-        Xml2Parser::from_stream(strong, self.unlimited_size, stream, cancellable)
-            .and_then(|parser| parser.parse())
-            .and_then(|_: ()| self.check_last_error())
+        Xml2Parser::from_stream(
+            strong,
+            self.load_options.unlimited_size,
+            stream,
+            cancellable,
+        )
+        .and_then(|parser| parser.parse())
+        .and_then(|_: ()| self.check_last_error())
     }
 
     fn unsupported_xinclude_start_element(&self, _name: &QualName) -> Context {
@@ -679,11 +742,11 @@ fn parse_xml_stylesheet_processing_instruction(data: &str) -> Result<Vec<(String
 
 pub fn xml_load_from_possibly_compressed_stream(
     document_builder: DocumentBuilder,
-    unlimited_size: bool,
+    load_options: Arc<LoadOptions>,
     stream: &gio::InputStream,
     cancellable: Option<&gio::Cancellable>,
 ) -> Result<Document, LoadingError> {
-    let state = Rc::new(XmlState::new(document_builder, unlimited_size));
+    let state = Rc::new(XmlState::new(document_builder, load_options));
 
     state.inner.borrow_mut().weak = Some(Rc::downgrade(&state));
 
